/*
	The Grosso Java Code Library. (C) 2002, William Grosso

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
	02111-1307  USA

	Questions or comments regarding this code can be sent to
	William Grosso (grosso@acm.org)

*/

package aspects.wgrosso.ddjarticle1;

import java.rmi.*;
import java.rmi.server.*;

import org.aspectj.lang.*;

/*
	Logs all calls that come into a method that throws a remote exception.
	Uses target pointcut type to make sure target is instance of UnicastRemoteObject
*/


public aspect Aspect_LogRemoteMethodCalls {

	pointcut potentiallyRemoteMethodCall():
		target(UnicastRemoteObject) &&
		call(public * * (..) throws RemoteException);

	pointcut comWGrossoCode():
		execution(public * com.wgrosso..*.*(..));

	pointcut reallyRemoteMethodCall():
		potentiallyRemoteMethodCall () &&
		!cflowbelow(comWGrossoCode());

	before(): reallyRemoteMethodCall() {
		logRemoteCall(thisJoinPoint);
	}

	private void logRemoteCall(JoinPoint joinPoint) {
		try {
			System.out.println("Remote call to " + joinPoint.getTarget());
			System.out.println("\tCaller is " + RemoteServer.getClientHost());
			System.out.println("\tAguments are: " + joinPoint.getArgs());
		}
		catch (Throwable t) {
		/*
			Just in case.
		*/
			t.printStackTrace();
		}
	}
}