/*
	The Grosso Java Code Library. (C) 2002, William Grosso

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
	02111-1307  USA

	Questions or comments regarding this code can be sent to
	William Grosso (grosso@acm.org)

*/

package com.wgrosso.ddjarticle1;

import java.util.*;

/*
	InstanceCreator.

	Creates and destroys instances. Basically a demo of the memory tracking
	aspect. This tracks instances of A's and B's because it knows about them
	(and because they have memory tracking code inserted into them as 
	statics). The memory tracking aspect does the same thing, without
	needing to insert code into specific classes. 
*/

public class InstanceCreator {

	private static long FIVE_SECONDS = 5 * 1000;
	private static long TIME_DELAY = FIVE_SECONDS ;
	private static LinkedList instances = new LinkedList();

	public static void main(String[] args) {
		while (true) {
			performRandomCreation();
			sleepForABit();
			System.out.println();
			performRandomDestruction();
			sleepForABit();
			System.out.println();
		}
	}

	private static void sleepForABit() {
		try {
			Thread.sleep(TIME_DELAY );
		}
		catch(Exception e) {};
	}

	private static void performRandomCreation() {
		int numberOfObjectsToCreate = getSmallSomewhatRandomNumber();
		for(int i =0; i < numberOfObjectsToCreate; i++) {
			Object newlyCreatedObject;
			if (Math.random() > .5) {
				newlyCreatedObject = new A();
			}
			else {
				newlyCreatedObject = new B();
			}
			instances.add(newlyCreatedObject);
		}	
	}


	private static void performRandomDestruction() {
		int numberOfObjectsToDestroy = getSmallSomewhatRandomNumber();
		for(int i =0; i < numberOfObjectsToDestroy; i++) {
			if (0==instances.size()) {
				return;
			}
			instances.removeFirst();
			System.gc();
		}
	}
	
	private static int getSmallSomewhatRandomNumber() {
		double base = Math.random();
		if (base <.1) {
			return 10;
		}
		else {
			return (int) ( (1/base) + 1);
		}
	}

	private static class A {
		private static int _numberOfInstances;

		private static synchronized void incrementNumberOfInstances() {
			_numberOfInstances++;
			System.out.println("Creating an instance of A. Currently, there are " 
				+ _numberOfInstances + " instances of A");
		}

		private static synchronized void decrementNumberOfInstances() {
			_numberOfInstances--;
			System.out.println("Finalizing an instance of A. Currently, there are " 
				+ _numberOfInstances + " instances of A");
		}

		public A() {
			incrementNumberOfInstances();
		}

		public void finalize() throws Throwable {
			decrementNumberOfInstances();	
			super.finalize();
		}
	}

	private static class B {
		private static int _numberOfInstances;

		private static synchronized void incrementNumberOfInstances() {
			_numberOfInstances++;
			System.out.println("Creating an instance of B. Currently, there are " 
				+ _numberOfInstances + " instances of B");
		}

		private static synchronized void decrementNumberOfInstances() {
			_numberOfInstances--;
			System.out.println("Finalizing an instance of B. Currently, there are " 
				+ _numberOfInstances + " instances of B");
		}

		public B() {
			incrementNumberOfInstances();
		}

		public void finalize() throws Throwable {
			decrementNumberOfInstances();	
			super.finalize();
		}
	}


}