// (C) 2002 MetaCase Consulting
// The basic watch behaviour. Handles state machine stack and event dispatching, and alarms. 
// Actual individual state machines are specified in generated classes, generic state machine behaviour in AbstractWatchApplication

import java.util.*;

public class Master

{
	AbstractWatchApplet applet;
	AbstractDisplay display;
	Vector alarms = new Vector();
	Hashtable applications = new Hashtable();
	Stack appStack = new Stack();
	METime clockOffset = new METime();
	int blinkingUnit = -1;

	public void init(AbstractWatchApplet applet, AbstractDisplay display, AbstractWatchApplication topApp)
	{
		this.applet=applet;
		this.display=display;
		addThisAndSubApps(topApp.appName(), topApp);
		startWatchApp(topApp.appName());
	}

	public void setBlinks(int unit)
	{
		blinkingUnit = unit;
	}

	public void sendEvent(String eventName)
	{
		((AbstractWatchApplication)(appStack.peek())).handleEvent(eventName);
	}

	public void jog()
	{
		sendEvent("");
	}

	public void addThisAndSubApps(String appName, AbstractWatchApplication app)
	{
		applications.put(appName, app);
		try
		{
			for (Enumeration e = app.decompositions.elements() ; e.hasMoreElements() ;)
			{
				DecompPair decompPair = (DecompPair)(e.nextElement());
				addThisAndSubApps(decompPair.name, decompPair.application);
			}
		}
		catch (Exception e) {}
	}

	public void startWatchApp(String appName)
	{
		try
		{
			appStack.push((AbstractWatchApplication)(applications.get(appName)));
			jog();
		}
		catch (Exception e) {}
	}

	public void clockOffsetChanged(METime offsetChange)
	{
		Alarm oldAlarm;
		Alarm newAlarm;
		METime newTime;

		for (int i=0; i<alarms.size() ; i++)
		{
			oldAlarm=(Alarm)(alarms.elementAt(i));
			if(oldAlarm.localTimeAwareness)
			{
				newTime=new METime();
				newTime.add(METime.HUNDREDTH, oldAlarm.sleeptime/10);
				newTime=newTime.meMinus(offsetChange);
				newAlarm=new Alarm(oldAlarm.name, oldAlarm.localTimeAwareness, oldAlarm.alarmApplication, oldAlarm.alarmState, newTime, this);
				alarms.setElementAt(newAlarm, i);
				newAlarm.start();
				oldAlarm.stopAlarm();
			}
		}
	}

	public void killAlarm(Alarm alarm)
	{
		alarms.removeElement(alarm);
		alarm.stopAlarm();
	}

	public void raiseAlarm(Alarm alarm)
	{
		alarm.alarmApplication.currentState=alarm.alarmState;
		if (((AbstractWatchApplication)(appStack.peek()))==alarm.alarmApplication)
			jog();
		else
			startWatchApp(alarm.alarmApplication.appName());
		killAlarm(alarm);
		applet.ringAlarm();
	}

}


