/*
	The Grosso Java Code Library. (C) 2002, William Grosso

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
	02111-1307  USA

	Questions or comments regarding this code can be sent to
	William Grosso (grosso@acm.org)

*/

package com.wgrosso.ddjarticle1;


/*
	PrintArguments is a simple program that prints out the 
	command line arguments. 

	The point is that the following lines are interesting:

		Object stringWrapper = new StringWrapper(args[i]);
		System.out.println("\t" + stringWrapper.toString() + "\n");

 	The declared type of stringWrapper is Object, while its
	runtime type is StringWrapper. This means that call and
	execution pointcuts could behave differently here (since 
	call works based on the declared type and execution 
	works based on the runtime type). 

*/

public class PrintArguments {

	public static void main(String[] args) {
		if ((null==args) || (0==args.length)) {
			System.out.println("No arguments");
		}
		else {
			System.out.println("Arguments:\n");
			for (int i=0; i < args.length; i++) {
				Object stringWrapper = new StringWrapper(args[i]);
				System.out.println("\t" + stringWrapper.toString() + "\n");
			}
		}
	}

	public static class StringWrapper {
		private String _string;

		public StringWrapper(String string) {
			_string = string;
		}

		public String toString() {
			System.out.println("In to string");
			return _string;
		}
	}
}